"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
function _actions() {
  const data = _interopRequireDefault(require("@nocobase/actions"));
  _actions = function _actions() {
    return data;
  };
  return data;
}
function _utils() {
  const data = require("@nocobase/utils");
  _utils = function _utils() {
    return data;
  };
  return data;
}
var _constants = require("../../constants");
var _jobs = _interopRequireDefault(require("./collecions/jobs"));
var _users = _interopRequireDefault(require("./collecions/users"));
var _users_jobs = _interopRequireDefault(require("./collecions/users_jobs"));
var _actions2 = require("./actions");
var _forms = _interopRequireDefault(require("./forms"));
const _excluded = ["mode"];
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }
function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }
function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }
function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }
const MULTIPLE_ASSIGNED_MODE = {
  SINGLE: Symbol('single'),
  ALL: Symbol('all'),
  ANY: Symbol('any'),
  ALL_PERCENTAGE: Symbol('all percentage'),
  ANY_PERCENTAGE: Symbol('any percentage')
};
const Modes = {
  [MULTIPLE_ASSIGNED_MODE.SINGLE]: {
    getStatus(distribution, assignees) {
      const done = distribution.find(item => item.status !== _constants.JOB_STATUS.PENDING && item.count > 0);
      return done ? done.status : null;
    }
  },
  [MULTIPLE_ASSIGNED_MODE.ALL]: {
    getStatus(distribution, assignees) {
      const resolved = distribution.find(item => item.status === _constants.JOB_STATUS.RESOLVED);
      if (resolved && resolved.count === assignees.length) {
        return _constants.JOB_STATUS.RESOLVED;
      }
      const rejected = distribution.find(item => item.status < _constants.JOB_STATUS.PENDING);
      if (rejected && rejected.count) {
        return rejected.status;
      }
      return null;
    }
  },
  [MULTIPLE_ASSIGNED_MODE.ANY]: {
    getStatus(distribution, assignees) {
      const resolved = distribution.find(item => item.status === _constants.JOB_STATUS.RESOLVED);
      if (resolved && resolved.count) {
        return _constants.JOB_STATUS.RESOLVED;
      }
      const rejectedCount = distribution.reduce((count, item) => item.status < _constants.JOB_STATUS.PENDING ? count + item.count : count, 0);
      // NOTE: all failures are considered as rejected for now
      if (rejectedCount === assignees.length) {
        return _constants.JOB_STATUS.REJECTED;
      }
      return null;
    }
  }
};
function getMode(mode) {
  switch (true) {
    case mode === 1:
      return Modes[MULTIPLE_ASSIGNED_MODE.ALL];
    case mode === -1:
      return Modes[MULTIPLE_ASSIGNED_MODE.ANY];
    case mode > 0:
      return Modes[MULTIPLE_ASSIGNED_MODE.ALL_PERCENTAGE];
    case mode < 0:
      return Modes[MULTIPLE_ASSIGNED_MODE.ANY_PERCENTAGE];
    default:
      return Modes[MULTIPLE_ASSIGNED_MODE.SINGLE];
  }
}
class _default {
  constructor(plugin) {
    this.plugin = void 0;
    this.formTypes = new (_utils().Registry)();
    this.plugin = plugin;
    plugin.db.collection(_users_jobs.default);
    plugin.db.extendCollection(_users.default);
    plugin.db.extendCollection(_jobs.default);
    plugin.app.resource({
      name: 'users_jobs',
      actions: {
        list: {
          filter: {
            $or: [{
              'workflow.enabled': true
            }, {
              'workflow.enabled': false,
              status: {
                $ne: _constants.JOB_STATUS.PENDING
              }
            }]
          },
          handler: _actions().default.list
        },
        submit: _actions2.submit
      }
    });
    (0, _forms.default)(this);
  }
  run(node, prevJob, processor) {
    return _asyncToGenerator(function* () {
      var _prevJob$id;
      const _node$config = node.config,
        mode = _node$config.mode,
        config = _objectWithoutProperties(_node$config, _excluded);
      const assignees = [...new Set(processor.getParsedValue(config.assignees) || [])];
      const job = yield processor.saveJob({
        status: _constants.JOB_STATUS.PENDING,
        result: mode ? [] : null,
        nodeId: node.id,
        upstreamId: (_prevJob$id = prevJob === null || prevJob === void 0 ? void 0 : prevJob.id) !== null && _prevJob$id !== void 0 ? _prevJob$id : null
      });
      // NOTE: batch create users jobs
      const UserJobModel = processor.options.plugin.db.getModel('users_jobs');
      yield UserJobModel.bulkCreate(assignees.map(userId => ({
        userId,
        jobId: job.id,
        nodeId: node.id,
        executionId: job.executionId,
        workflowId: node.workflowId,
        status: _constants.JOB_STATUS.PENDING
      })), {
        transaction: processor.transaction
      });
      return job;
    })();
  }
  resume(node, job, processor) {
    return _asyncToGenerator(function* () {
      var _getMode$getStatus, _job$latestUserJob$re, _job$latestUserJob;
      // NOTE: check all users jobs related if all done then continue as parallel
      const _node$config2 = node.config,
        _node$config2$assigne = _node$config2.assignees,
        assignees = _node$config2$assigne === void 0 ? [] : _node$config2$assigne,
        mode = _node$config2.mode;
      const UserJobModel = processor.options.plugin.db.getModel('users_jobs');
      const distribution = yield UserJobModel.count({
        where: {
          jobId: job.id
        },
        group: ['status'],
        transaction: processor.transaction
      });
      const submitted = distribution.reduce((count, item) => item.status !== _constants.JOB_STATUS.PENDING ? count + item.count : count, 0);
      const status = job.status || ((_getMode$getStatus = getMode(mode).getStatus(distribution, assignees)) !== null && _getMode$getStatus !== void 0 ? _getMode$getStatus : _constants.JOB_STATUS.PENDING);
      const result = mode ? (submitted || 0) / assignees.length : (_job$latestUserJob$re = (_job$latestUserJob = job.latestUserJob) === null || _job$latestUserJob === void 0 ? void 0 : _job$latestUserJob.result) !== null && _job$latestUserJob$re !== void 0 ? _job$latestUserJob$re : job.result;
      processor.logger.debug(`manual resume job and next status: ${status}`);
      job.set({
        status,
        result
      });
      return job;
    })();
  }
}
exports.default = _default;